#include <string>

#import <Foundation/Foundation.h>
#import "PokktManager.h"
#import "PokktController.h"
#import "VideoResponse.h"

// game object name
#define UnityListenerGOName             "pokktDispatcher"

// events
#define VideoDisplayedEvent             "VideoDisplayed"
#define VideoSkippedEvent               "VideoSkipped"
#define VideoClosedEvent                "VideoClosed"
#define VideoCompletedEvent             "VideoCompleted"
#define VideoGratifiedEvent             "VideoGratified"
#define DownloadCompletedEvent          "DownloadCompleted"
#define DownloadFailedEvent             "DownloadFailed"

PokktManager *instance;

@interface PokktNativeExtension:NSObject<PokktDelegate>
@end

extern "C"
{
    void NotifyIOS(const char* operation, const char* param)
    {
        NSString *paramNSStr = [NSString stringWithUTF8String:(param)];
        
        if (std::strcmp(operation, "setParams") == 0)
        {
            NSArray *paramArray = [paramNSStr componentsSeparatedByString:@","];
            if (paramArray != nil && paramArray.count == 5)
                instance = [PokktManager
                            sharedInstance:@{
                                             SECURITY_KEY:[paramArray objectAtIndex:0],
                                             APPLICATION_ID:[paramArray objectAtIndex:1],
                                             USER_ID:[paramArray objectAtIndex:2],
                                             INTEGRATION_TYPE:[paramArray objectAtIndex:3],
                                             AUTO_CACHE_VIDEO:[NSNumber numberWithBool:[[paramArray objectAtIndex:4] boolValue]]
                                             }];
            else
                instance = [PokktManager sharedInstance];
            
            PokktNativeExtension *ext = [[PokktNativeExtension alloc] init];
            [instance setPokktDelegate:ext];
        }
        
        else if (std::strcmp(operation, "getVideo") == 0)
        {
            // set the view controller
            UIViewController *viewController = [[[UIApplication sharedApplication] keyWindow] rootViewController];
            [instance setPresentView:viewController];
            [instance playVideoCampaign:YES withScreenName:paramNSStr];
        }
        
        else if (std::strcmp(operation, "getVideoNonIncent") == 0)
        {
            // set the view controller
            UIViewController *viewController = [[[UIApplication sharedApplication] keyWindow] rootViewController];
            [instance setPresentView:viewController];
            [instance playVideoCampaign:NO withScreenName:paramNSStr];
        }
        
        else if (std::strcmp(operation, "getVideoWithIntent") == 0)
        {
            // set the view controller
            UIViewController *viewController = [[[UIApplication sharedApplication] keyWindow] rootViewController];
            [instance setPresentView:viewController];
            [instance playVideoCampaign:YES withScreenName:paramNSStr];
        }
        
        else if (std::strcmp(operation, "startVideoCaching") == 0)
            [instance cacheVideoCampaign];
        
        else if (std::strcmp(operation, "setSkipVideo") == 0)
            [instance setSkipVideo:(std::strcmp(param, "true") == 0)];
        
        else if (std::strcmp(operation, "setDefaultSkipTime") == 0)
            [instance setDefaultSkipTime:[paramNSStr intValue]];
        
        else if (std::strcmp(operation, "setAge") == 0)
            [instance setAge:paramNSStr];
        
        else if (std::strcmp(operation, "setBirthday") == 0)
            [instance setBirthday:paramNSStr];
        
        else if (std::strcmp(operation, "setContentType") == 0)
            [instance setContent_type:paramNSStr];
        
        else if (std::strcmp(operation, "setEducationType") == 0)
            [instance setEducation_Information:paramNSStr];
        
        else if (std::strcmp(operation, "setEmploymentStatus") == 0)
            [instance setEmployment_Status:paramNSStr];
        
        else if (std::strcmp(operation, "setFacebookId") == 0)
            [instance setFacebook_ID:paramNSStr];
        
        else if (std::strcmp(operation, "setLocation") == 0)
            [instance setLocation:paramNSStr];
        
        else if (std::strcmp(operation, "setMaritalStatus") == 0)
            [instance setMartial_Status:paramNSStr];
        
        else if (std::strcmp(operation, "setName") == 0)
            [instance setName:paramNSStr];
        
        else if (std::strcmp(operation, "setNationality") == 0)
            [instance setNationality:paramNSStr];
        
        else if (std::strcmp(operation, "setSex") == 0)
            [instance setSex:paramNSStr];
        
        else if (std::strcmp(operation, "setTwitterHandle") == 0)
            [instance setTwitter_Handle:paramNSStr];
        
        else if (std::strcmp(operation, "setThePubs") == 0)
        {
            NSArray *paramArray = [paramNSStr componentsSeparatedByString:@","];
            [instance setPubs:paramArray];
        }
        
        else if (std::strcmp(operation, "setDebug") == 0)
        {
            BOOL debug = (std::strcmp(param, "true") == 0);
            [PokktManager setDebugger:debug];
        }
    }
    
    bool IsVideoAvailableOnIOS()
    {
        return [instance isVideoAvailable];
    }
    
    double GetVideoVCOnIOS()
    {
        double vc = [instance.videoVC doubleValue];
        return vc;
    }
    
    const char* GetPokktSDKVersionOnIOS()
    {
        const char* version = [[NSString stringWithFormat:@"%@", instance.SDKVersion] UTF8String];
        return version;
    }
}

// use this to notify the unity observer
static void NotifyUnity(const char* operation, const char*  param)
{
    UnitySendMessage(UnityListenerGOName, operation, param);
}

@implementation PokktNativeExtension

#pragma mark - Pokkt Delegates

- (void)didFinishedVideoDownload:(BOOL)isFinished
{
    NotifyUnity(DownloadCompletedEvent, [[NSString stringWithFormat:@"%@", instance.videoVC] UTF8String]);
}

- (void)didFailedVideoDownload:(NSString *)error
{
    NotifyUnity(DownloadFailedEvent, [[NSString stringWithFormat:@"%@", error] UTF8String]);
}

- (void)onVideoCompleated:(BOOL)isCompleated
{
    NotifyUnity(VideoCompletedEvent, "Video is completed");
}

- (void)onVideoDisplayed
{
    NotifyUnity(VideoDisplayedEvent, "Video is displayed");
}

- (void)onVideoClosed
{
    NotifyUnity(VideoClosedEvent, "Video is closed");
}

- (void)onVideoGratified:(VideoResponse *)videoResponse
{
    NotifyUnity(VideoGratifiedEvent, [[NSString stringWithFormat:@"%@", videoResponse.coins] UTF8String]);
}

- (void)onVideoSkiped
{
    NotifyUnity(VideoSkippedEvent, "Video is skipped");
}

@end