﻿using UnityEngine;
using System;
using System.Collections;
using System.Runtime.InteropServices;

namespace Pokkt
{
	internal static class PokktNativeExtension
	{
		private static IPerformer _performer;

		public static void InitPerformer()
		{
			if (Application.platform == RuntimePlatform.Android)
				_performer = new AndroidPerformer();
			
			else if (Application.platform == RuntimePlatform.IPhonePlayer)
				_performer = new IOSPerformer();

			else
				Debug.LogWarning("[UNITY] Performer not implemented for platform: " + Application.platform.ToString());
		}

		public static void PerformOperation(PokktOperations operation, string param = "")
		{
			switch (operation)
			{
				case PokktOperations.SetParams:                 NotifyNative("setParams",               param);   break;
				case PokktOperations.SetDebug:                  NotifyNative("setDebug",                param);   break;
				case PokktOperations.ShowLog:                   NotifyNative("showLog",                 param);   break;
				case PokktOperations.ShowToast:                 NotifyNative("showToast",               param);   break;
					
				case PokktOperations.GetFreeCoins:              NotifyNative("getFreeCoins",            param);   break;
				case PokktOperations.GetPendingCoins:           NotifyNative("getPendingCoins",     	param);   break;
				case PokktOperations.CheckOfferwallCampaign:    NotifyNative("checkOfferwallCampaign",  param);   break;
				case PokktOperations.GetVideo:                  NotifyNative("getVideo",            	param);   break;
				case PokktOperations.GetVideoNonIncent:    		NotifyNative("getVideoNonIncent",   	param);   break;
				case PokktOperations.GetVideoWithIntent:    	NotifyNative("getVideoWithIntent",      param);   break;
					
				case PokktOperations.StartVideoCaching:         NotifyNative("startVideoCaching",       param);   break;
				case PokktOperations.SetSkipVideo:          	NotifyNative("setSkipVideo",        	param);   break;
				case PokktOperations.SetDefaultSkipTime:    	NotifyNative("setDefaultSkipTime",      param);   break;
				case PokktOperations.SetAge:                    NotifyNative("setAge",              	param);   break;
				case PokktOperations.SetBirthday:               NotifyNative("setBirthday",             param);   break;
				case PokktOperations.SetContentType:            NotifyNative("setContentType",          param);   break;
				case PokktOperations.SetEducationType:          NotifyNative("setEducationType",    	param);   break;
				case PokktOperations.SetEmploymentStatus:       NotifyNative("setEmploymentStatus",     param);   break;
				case PokktOperations.SetFacebookId:             NotifyNative("setFacebookId",           param);   break;
				case PokktOperations.SetLocation:               NotifyNative("setLocation",             param);   break;
				case PokktOperations.SetMaritalStatus:          NotifyNative("setMaritalStatus",        param);   break;
				case PokktOperations.SetName:                   NotifyNative("setName",                 param);   break;
				case PokktOperations.SetNationality:            NotifyNative("setNationality",          param);   break;
				case PokktOperations.SetSex:                    NotifyNative("setSex",              	param);   break;
				case PokktOperations.SetThePubs:                NotifyNative("setThePubs",              param);   break;
				case PokktOperations.SetThirdPartyUserId:       NotifyNative("setThirdPartyUserId",     param);   break;
				case PokktOperations.SetTwitterHandle:          NotifyNative("setTwitterHandle",        param);   break;
			}
		}

		private static void NotifyNative(string operation, string param)
		{
			Debug.Log("[UNITY] Performing operation: " + operation + " with params: " + param);
			if (_performer != null)
				_performer.NotifyNative(operation, param);
		}

		public static bool IsVideoAvailable()
		{
			Debug.Log("[UNITY] Checking video availability...");
			if (_performer != null)
				return _performer.IsVideoAvailable();

			return false;
		}
		
		public static float GetVideoVC()
		{
			Debug.Log("[UNITY] Getting video vc ...");
			if (_performer != null)
				return _performer.GetVideoVC();

			return -1;
		}
		
		public static string GetPokktSDKVersion()
		{
			Debug.Log("[UNITY] Getting Pokkt SDK version ...");
			if (_performer != null)
				return _performer.GetPokktSDKVersion();

			return "";
		}
	}
}