﻿using UnityEngine;
using System.Collections;
using System;

namespace Pokkt
{
	public class PokktManager
	{
		private static PokktManager _instance = null;

		public static PokktManager GetInstance()
		{
			if (_instance == null) 
				_instance = new PokktManager();

			return _instance; 
		}

		// dispatcher name
		private const string DispatcherName = "pokktDispatcher";

		// pokkt params
		public static string SecurityKey = "";
		public static string ApplicationId = "";
		public static string UserId = "";
		public static string IntegrationType = ""; // FIXME: should not be empty
		private static string AutoCacheVideo = ""; // FIXME: should not be empty

		// internal state
		private GameObject _pokktDispatcher;

		private PokktManager()
		{
			PokktNativeExtension.InitPerformer();

			CheckAndInitDispatcher();
		}

		private void CheckAndInitDispatcher()
		{
			// check for any existing dispatcher go
			_pokktDispatcher = GameObject.Find(DispatcherName);

			// if it does not exists, create one
			if (_pokktDispatcher == null)
			{
				// create a game object to listen to messages from java
				_pokktDispatcher = new GameObject(DispatcherName);

				// retain this object
				GameObject.DontDestroyOnLoad(_pokktDispatcher);
			}

			// attach the dispatcher sript component to it, if not available already
			if (_pokktDispatcher.GetComponent<PokktDispatcher>() == null)
				_pokktDispatcher.AddComponent<PokktDispatcher>();
		}

		public bool AutoCaching
		{
			set
			{
				AutoCacheVideo = (value ? "true" : "false");
				SetParams();
			}
			get { return (AutoCacheVideo == "true"); }
		}

		public PokktDispatcher Dispathcer
		{
			get
			{
				// keep checking for dispatcher
				CheckAndInitDispatcher();
				
				return _pokktDispatcher.GetComponent<PokktDispatcher>();
			}
		}

		public void SetParams() // TODO: rename to InitPokkt
		{
			string paramString = SecurityKey + "," + ApplicationId + "," + UserId + "," + IntegrationType + "," + AutoCacheVideo;
			PokktNativeExtension.PerformOperation(PokktOperations.SetParams, ((AutoCacheVideo == "true") ? "" : paramString));
		}

		public void SetDebug(bool value)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.SetDebug, (value ? "true" : "false"));
		}

		public void ShowLog(string message)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.ShowLog, message);
		}

		public void ShowToast(string message)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.ShowToast, message);
		}
		
		public void GetFreeCoins()
		{
			GetFreeCoins("0");
		}

		public void GetFreeCoins(string assetValue)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.GetFreeCoins, assetValue);
		}

		public void GetPendingCoins()
		{
			PokktNativeExtension.PerformOperation(PokktOperations.GetPendingCoins);
		}

		public void CheckOfferwallCampaign()
		{
			PokktNativeExtension.PerformOperation(PokktOperations.CheckOfferwallCampaign);
		}

		public void GetVideo(string screenName)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.GetVideo, screenName);
		}

		public void GetVideoNonIncent(string screenName)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.GetVideoNonIncent, screenName);
		}

		public void GetVideoWithIntent(string screenName)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.GetVideoWithIntent, screenName);
		}

		public void StartVideoCaching()
		{
			PokktNativeExtension.PerformOperation(PokktOperations.StartVideoCaching);
		}

		public void SetSkipVideo(bool canSkip)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.SetSkipVideo, (canSkip ? "true" : "false"));
		}

		public void SetDefaultSkipTime(string time)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.SetDefaultSkipTime, time);
		}

		public void SetAge(string age)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.SetAge, age);
		}

		public void SetBirthday(string birthday)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.SetBirthday);
		}

		public void SetContentType(string contentType)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.SetContentType, contentType);
		}

		public void SetEducationType(string educationType)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.SetEducationType, educationType);
		}

		public void SetEmploymentStatus(string empStatus)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.SetEmploymentStatus, empStatus);
		}

		public void SetFacebookId(string fbId)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.SetFacebookId, fbId);
		}

		public void SetLocation(string location)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.SetLocation, location);
		}

		public void SetMaritalStatus(string maritalStatus)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.SetMaritalStatus, maritalStatus);
		}

		public void SetName(string name)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.SetName, name);
		}

		public void SetNationality(string nationality)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.SetNationality, nationality);
		}

		public void SetSex(string sex)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.SetSex, sex);
		}

		public void SetThePubs(string pubs)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.SetThePubs, pubs);
		}

		public void SetThirdPartyUserId(string thirdPartyUserId)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.SetThirdPartyUserId, thirdPartyUserId);
		}

		public void SetTwitterHandle(string twitterHandle)
		{
			PokktNativeExtension.PerformOperation(PokktOperations.SetTwitterHandle, twitterHandle);
		}

		public bool IsVideoAvailable()
		{
			return PokktNativeExtension.IsVideoAvailable();
		}

		public float GetVideoVc()
		{
			return PokktNativeExtension.GetVideoVC();
		}
		
		public static string GetPokktSDKVersion()
		{
			return PokktNativeExtension.GetPokktSDKVersion();
		}
	}
}